Hi, thanks for reading and downloading the replication package for "Characterizing Software User Feedback Text Clusters" from Requirements Engineering 2022.

This package should help you replicate the results of our paper.

# Summary of Artifact

In this replication package, we have two folders, `/method` and `/eval`.

### Requirements

This project was created and tested in Python 3.8. Other versions of Python 3 may work, but we recommend using 3.8.

Requirements can be found in the `requirements.txt` or `environment.yml` files in the root of this project.

The two should ultimately give you the same result in terms of packages available.
The `requirements.txt` file should work with pip like so:
```bash
pip install -r requirements.txt
```
from the root folder of this project. This simply installs all libraries using pip to your current environment.

The `environment.yml` is designed to recreate the entire conda environment. This requires that conda is installed, and is run like so from the root dir:
```bash
conda deactivate
conda env create -f environment.yml
conda activate new_cluster_char_env
```
This creates a new conda environment with a specific Python version and activates it.

There may be some issues with installing icu on Windows machines. I fixed this on my machine by following the advice here: https://stackoverflow.com/a/58132293
This fix involves downloading PyICU and PyCLD2 from https://www.lfd.uci.edu/~gohlke/pythonlibs/ which are relevant for your Python version.
For example, downloading `pycld2‑0.41‑cp38‑cp38‑win_amd64.whl` and `PyICU‑2.9‑cp38‑cp38‑win_amd64.whl`.
These can then be installed using:
```powershell
python -m pip install C:\Path\To\Download\pycld2‑0.41‑cp38‑cp38‑win_amd64.whl
python -m pip install C:\Path\To\Download\PyICU‑2.9‑cp38‑cp38‑win_amd64.whl
```

The packages included in these files include:
* notebook (to run the ipynb)
* pandas
* icu
* stopwordsiso
* zipfile
* requests
* beautifulsoup4
* scipy
* langdetect
* sqlite3
* hdbscan
* umap
* numpy
* pickle
* tqdm
* tensorflow_hub
* nltk
* autocorrect
* torch


### What the /method code does

The `/method` folder contains the code to:
* Download the raw user feedback required for this paper.
* Embed this feedback using the USE model.
* Reduce dimensionality using UMAP.
* Cluster this feedback into coherent clusters using HDBSCAN.
* Create characterizations for each cluster.
* Output a random sample of these characterizations for use in the labelling task (described in the paper).

The data output from this step is disconnected from the `/eval` folder.
One needs to label the data and join it together for it to then be analysed in the `/eval` folder.

### How to run the /method code
Navigate to the `/method` folder, then run:
`python3 run_characterization_units_expt.py`

### What the /eval code does
The `/eval` folder contains the data generated from `/method`, but labelled by the labellers as described in the paper.
This folder also contains the code to visualise the results and get all the results presented within the paper.

### How to run the /eval code
Run the panels in `eval_data_analysis.ipynb` by running a jupyter notebook (I.e. running `jupyter notebook` in the root directory).

# How to use this code on your own data

A ipynb of how to use this code with your own data can be found at `/method/use_your_own_data.ipynb`.

Example:
```python
from characterization_units_expt.clustering_algos.hdbscan import HdbscanClusterer
from characterization_units_expt.text_embed import TextEmbedder
from characterization_units_expt.cluster_prepper import cluster_df
import pandas as pd

# You can play around with these parameters if you want smaller or bigger clusters
min_cluster_size = 30
umap_nn = 100

my_text_list = [
# YOUR LIST OF FEEDBACK STRINGS HERE
]

my_df = pd.DataFrame({"text": my_text_list})

text_embedder = TextEmbedder(language_code = "en")

my_embeds = text_embedder.embed_text(my_df.text.tolist())

clusterer = HdbscanClusterer(min_cluster_size=min_cluster_size, umap_nn=umap_nn)
cluster_char_df = cluster_df(my_df, my_embeds, text_embedder, clusterer)
```

# Authors Information

This artifact relates to

Peter Devine, James Tizard, Hechen Wang, Yun Sing Koh, Kelly Blincoe "Characterizing Software User Feedback Text Clusters" 2022 IEEE 30th International Requirements Engineering Conference (RE). IEEE, 2022.

Please cite it if you use anything from here :)

# Artifact Location

This artifact is hosted on Zenodo at https://zenodo.org/record/6562615
