import hdbscan
# from sklearn.metrics import pairwise_distances
# from sklearn.preprocessing import normalize
import umap

class HdbscanClusterer:
    def __init__(self, min_cluster_size, umap_nn):        
        self.clusterer = hdbscan.HDBSCAN(algorithm='boruvka_kdtree',
                            cluster_selection_method ='leaf', 
                            min_cluster_size=min_cluster_size, 
                            metric='euclidean', 
                            min_samples=None)
        self.reducer = umap.UMAP(n_neighbors=umap_nn, n_components=20, metric='cosine', output_metric='euclidean', min_dist=0)
            
    def fit(self, embeddings):
        print("Normalizing")
        print(embeddings.shape)
        lo_dim_embeddings = self.reducer.fit_transform(embeddings)
        
#         distance = pairwise_distances(embeddings, metric='cosine')
        print("Fitting")
        self.clusterer.fit(lo_dim_embeddings)
        return self.clusterer
    
    def get_cluster_ids(self, distance=None):
        if distance is None:
            return self.clusterer.labels_
        else:
            return self.clusterer.single_linkage_tree_.get_clusters(distance, min_cluster_size=2)