import pandas as pd
from langdetect import detect
from bs4 import BeautifulSoup

def download_spotify_twitter():
    df = pd.read_csv("./temp_data/spotify_2021/twitter_scraper.csv")

    df["label"] = None
    df["app_name"] = "spotify_twitter"

    return df

def download_spotify_apple():
    df = pd.read_csv("./temp_data/spotify_2021/apple_scraper.csv")

    df["label"] = None
    df["app_name"] = "spotify_apple"
    df["text"] = df.title + ". " + df.content

    return df

def download_spotify_play():
    df = pd.read_csv("./temp_data/spotify_2021/play_scraper.csv")

    df["label"] = None
    df["app_name"] = "spotify_play"
    df["text"] = df.review_text

    return df

def download_spotify_reddit():
    df = pd.read_csv("./temp_data/spotify_2021/reddit_scraper.csv")

    df["label"] = None
    df["app_name"] = "spotify_reddit"
    df["text"] = df.title + ". " + df.body

    return df

# Remove specs text (E.g. Device text, OS info) from forum posts because they do not describe requirements.
def get_clean_text(post_html):
    soup = BeautifulSoup(post_html, "html.parser")

    is_specs = False
    all_texts = []
    for p_element in soup.find_all("p"):

        el_text = p_element.get_text()
        is_specs_header = el_text in ["Device", "Plan", "Country", "Operating System"]

        if not is_specs and not is_specs_header:
            if el_text != "My Question or Issue" and len(el_text.strip()) > 0:
                all_texts.append(el_text)

        is_specs = is_specs_header

    return ". \n".join([x for x in all_texts])

def download_spotify_forum():
    df = pd.read_csv("./temp_data/spotify_2021/forum_scraper.csv")

    df["description_text"] = df.description.apply(get_clean_text)

    df["label"] = None
    df["app_name"] = "spotify_forum"
    df["text"] = df.title + ". " + df.description_text

    return df
