import pandas as pd
import numpy as np

def pile_df(sampled_df):
    char_cols = ["char_text", "char_uni", "char_bi", "char_tri"]
    
    data_list = []

    for col in char_cols:
        col_df = sampled_df[["sample_text", col, f"random_{col}", "dataset_name", "app_name", "cluster_size", "all_cluster_index"]]
        col_df["char_type"] = col
        col_df = col_df.rename({col: "char_A", f"random_{col}": "char_B"}, axis=1)
        data_list.append(col_df)
        
    all_sampled_df = pd.concat(data_list)
    
    return all_sampled_df

def randomly_swap_chars(row):
    label = np.random.sample() > 0.5
    
    row["is_A_match"] = label
    
    if not label:
        char_A = row["char_A"]
        char_B = row["char_B"]
        
        row["char_A"] = char_B
        row["char_B"] = char_A
    
    return row

def prepare_data():
    df = pd.read_json("all_cluster_char_text_char.json",  lines=True)
    df["cluster_size"] = df.all_cluster_text.str.len()
    df["all_cluster_index"] = df.index
    
    sampled_df = df.groupby(["dataset_name", "app_name"]).apply(lambda x: x.sample(n=min(10, x.shape[0])))
    sampled_df = sampled_df.reset_index(drop=True)
    
    all_sampled_df = pile_df(sampled_df)

    all_sampled_df = all_sampled_df.apply(randomly_swap_chars, axis=1)
    
    all_sampled_df["char_A"] = all_sampled_df["char_A"].str.join("\n")
    all_sampled_df["char_B"] = all_sampled_df["char_B"].str.join("\n")
    all_sampled_df["best_match_(A/B)"] = ""
    all_sampled_df["is_sample_text_req_rel_(Yes/No)"] = ""
    all_sampled_df["is_char_req_rel_(Yes/No)"] = ""
    
    all_sampled_df = all_sampled_df.reset_index(drop=True)
    all_sampled_df = all_sampled_df.reset_index(drop=False)
    all_sampled_df = all_sampled_df[["index", "sample_text", "char_A", "char_B", "best_match_(A/B)", "is_sample_text_req_rel_(Yes/No)", "is_char_req_rel_(Yes/No)", "dataset_name", "app_name", "cluster_size", "all_cluster_index", "char_type", "is_A_match"]]
    all_sampled_df.to_csv("cluster_char_units_both_splits_with_labels.csv", index=False)
    green_df = all_sampled_df.sample(frac=0.5)
    orange_df = all_sampled_df.drop(green_df.index).sample(frac=1.0)

    green_df.to_csv("cluster_char_units_green_with_labels.csv", index=False)
    orange_df.to_csv("cluster_char_units_orange_with_labels.csv", index=False)

    green_df.drop(["is_A_match", "char_type"], axis=1).to_csv("cluster_char_units_green.csv", index=False)
    orange_df.drop(["is_A_match", "char_type"], axis=1).to_csv("cluster_char_units_orange.csv", index=False)