import tensorflow_hub as hub
import numpy as np
from tqdm import trange

class TextEmbedder:
    def __init__ (self, language_code="en"):

        self.batch_size = 8
        if language_code == "en":
            self.model = hub.load("https://tfhub.dev/google/universal-sentence-encoder-large/5")
        else:
            self.model = hub.load("https://tfhub.dev/google/universal-sentence-encoder-multilingual-large/3")

    def embed_text(self, text_list):

        embeddings = []
        for i in trange(0, len(text_list), self.batch_size):
            embedding = self.model(text_list[i:i+self.batch_size]).numpy()
            embeddings.append(embedding)

        return np.vstack(embeddings)

def embed_data_dict(data_dict):

    text_embedder = TextEmbedder()

    emb_dict = {}

    for data_name, df in data_dict.items():
        emb = text_embedder.embed_text(df.text.tolist())

        emb_dict[data_name] = emb

    return emb_dict
